// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

// THIS FILE WAS AUTO-GENERATED BY ./ntcore/generate_topics.py. DO NOT MODIFY

#pragma once

#include <stdint.h>

#include <utility>
#include <span>
#include <string_view>
#include <vector>

#include "wpi/nt/NetworkTableType.hpp"
#include "wpi/nt/Topic.hpp"
#include "wpi/nt/ntcore_cpp.hpp"

namespace wpi::util {
template <typename T>
class SmallVectorImpl;
class json;
}  // namespace wpi

namespace wpi::nt {

class StringArrayTopic;

/**
 * NetworkTables StringArray subscriber.
 */
class StringArraySubscriber : public Subscriber {
 public:
  using TopicType = StringArrayTopic;
  using ValueType = std::vector<std::string>;
  using ParamType = std::span<const std::string>;
  using TimestampedValueType = TimestampedStringArray;


  StringArraySubscriber() = default;

  /**
   * Construct from a subscriber handle; recommended to use
   * StringArrayTopic::Subscribe() instead.
   *
   * @param handle Native handle
   * @param defaultValue Default value
   */
  StringArraySubscriber(NT_Subscriber handle, ParamType defaultValue)
      : Subscriber{handle},
        m_defaultValue{defaultValue.begin(), defaultValue.end()} {}

  /**
   * Get the last published value.
   * If no value has been published, returns the stored default value.
   *
   * @return value
   */
  ValueType Get() const {
    return Get(m_defaultValue);
  }

  /**
   * Get the last published value.
   * If no value has been published, returns the passed defaultValue.
   *
   * @param defaultValue default value to return if no value has been published
   * @return value
   */
  ValueType Get(ParamType defaultValue) const {
    return ::wpi::nt::GetStringArray(m_subHandle, defaultValue);
  }

  /**
   * Get the last published value along with its timestamp
   * If no value has been published, returns the stored default value and a
   * timestamp of 0.
   *
   * @return timestamped value
   */
  TimestampedValueType GetAtomic() const {
    return GetAtomic(m_defaultValue);
  }

  /**
   * Get the last published value along with its timestamp.
   * If no value has been published, returns the passed defaultValue and a
   * timestamp of 0.
   *
   * @param defaultValue default value to return if no value has been published
   * @return timestamped value
   */
  TimestampedValueType GetAtomic(ParamType defaultValue) const {
    return ::wpi::nt::GetAtomicStringArray(m_subHandle, defaultValue);
  }

  /**
   * Get an array of all value changes since the last call to ReadQueue.
   * Also provides a timestamp for each value.
   *
   * @note The "poll storage" subscribe option can be used to set the queue
   *     depth.
   *
   * @return Array of timestamped values; empty array if no new changes have
   *     been published since the previous call.
   */
  std::vector<TimestampedValueType> ReadQueue() {
    return ::wpi::nt::ReadQueueStringArray(m_subHandle);
  }

  /**
   * Get the corresponding topic.
   *
   * @return Topic
   */
  TopicType GetTopic() const;

 private:
  ValueType m_defaultValue;
};

/**
 * NetworkTables StringArray publisher.
 */
class StringArrayPublisher : public Publisher {
 public:
  using TopicType = StringArrayTopic;
  using ValueType = std::vector<std::string>;
  using ParamType = std::span<const std::string>;

  using TimestampedValueType = TimestampedStringArray;

  StringArrayPublisher() = default;

  /**
   * Construct from a publisher handle; recommended to use
   * StringArrayTopic::Publish() instead.
   *
   * @param handle Native handle
   */
  explicit StringArrayPublisher(NT_Publisher handle) : Publisher{handle} {}

  /**
   * Publish a new value.
   *
   * @param value value to publish
   * @param time timestamp; 0 indicates current NT time should be used
   */
  void Set(ParamType value, int64_t time = 0) {
    ::wpi::nt::SetStringArray(m_pubHandle, value, time);
  }

  /**
   * Publish a default value.
   * On reconnect, a default value will never be used in preference to a
   * published value.
   *
   * @param value value
   */
  void SetDefault(ParamType value) {
    ::wpi::nt::SetDefaultStringArray(m_pubHandle, value);
  }

  /**
   * Get the corresponding topic.
   *
   * @return Topic
   */
  TopicType GetTopic() const;
};

/**
 * NetworkTables StringArray entry.
 *
 * @note Unlike NetworkTableEntry, the entry goes away when this is destroyed.
 */
class StringArrayEntry final : public StringArraySubscriber,
                                  public StringArrayPublisher {
 public:
  using SubscriberType = StringArraySubscriber;
  using PublisherType = StringArrayPublisher;
  using TopicType = StringArrayTopic;
  using ValueType = std::vector<std::string>;
  using ParamType = std::span<const std::string>;

  using TimestampedValueType = TimestampedStringArray;

  StringArrayEntry() = default;

  /**
   * Construct from an entry handle; recommended to use
   * StringArrayTopic::GetEntry() instead.
   *
   * @param handle Native handle
   * @param defaultValue Default value
   */
  StringArrayEntry(NT_Entry handle, ParamType defaultValue)
      : StringArraySubscriber{handle, defaultValue},
        StringArrayPublisher{handle} {}

  /**
   * Determines if the native handle is valid.
   *
   * @return True if the native handle is valid, false otherwise.
   */
  explicit operator bool() const { return m_subHandle != 0; }

  /**
   * Gets the native handle for the entry.
   *
   * @return Native handle
   */
  NT_Entry GetHandle() const { return m_subHandle; }

  /**
   * Get the corresponding topic.
   *
   * @return Topic
   */
  TopicType GetTopic() const;

  /**
   * Stops publishing the entry if it's published.
   */
  void Unpublish() {
    ::wpi::nt::Unpublish(m_pubHandle);
  }
};

/**
 * NetworkTables StringArray topic.
 */
class StringArrayTopic final : public Topic {
 public:
  using SubscriberType = StringArraySubscriber;
  using PublisherType = StringArrayPublisher;
  using EntryType = StringArrayEntry;
  using ValueType = std::vector<std::string>;
  using ParamType = std::span<const std::string>;
  using TimestampedValueType = TimestampedStringArray;
  /** The default type string for this topic type. */
  static constexpr std::string_view TYPE_STRING = "string[]";

  StringArrayTopic() = default;

  /**
   * Construct from a topic handle; recommended to use
   * NetworkTableInstance::GetStringArrayTopic() instead.
   *
   * @param handle Native handle
   */
  explicit StringArrayTopic(NT_Topic handle) : Topic{handle} {}

  /**
   * Construct from a generic topic.
   *
   * @param topic Topic
   */
  explicit StringArrayTopic(Topic topic) : Topic{topic} {}

  /**
   * Create a new subscriber to the topic.
   *
   * <p>The subscriber is only active as long as the returned object
   * is not destroyed.
   *
   * @note Subscribers that do not match the published data type do not return
   *     any values. To determine if the data type matches, use the appropriate
   *     Topic functions.
   *
   * @param defaultValue default value used when a default is not provided to a
   *        getter function
   * @param options subscribe options
   * @return subscriber
   */
  [[nodiscard]]
  SubscriberType Subscribe(
      ParamType defaultValue,
      const PubSubOptions& options = DEFAULT_PUB_SUB_OPTIONS) {
    return StringArraySubscriber{
        ::wpi::nt::Subscribe(m_handle, NT_STRING_ARRAY, "string[]", options),
        defaultValue};
  }
  /**
   * Create a new subscriber to the topic, with specific type string.
   *
   * <p>The subscriber is only active as long as the returned object
   * is not destroyed.
   *
   * @note Subscribers that do not match the published data type do not return
   *     any values. To determine if the data type matches, use the appropriate
   *     Topic functions.
   *
   * @param typeString type string
   * @param defaultValue default value used when a default is not provided to a
   *        getter function
   * @param options subscribe options
   * @return subscriber
   */
  [[nodiscard]]
  SubscriberType SubscribeEx(
      std::string_view typeString, ParamType defaultValue,
      const PubSubOptions& options = DEFAULT_PUB_SUB_OPTIONS) {
    return StringArraySubscriber{
        ::wpi::nt::Subscribe(m_handle, NT_STRING_ARRAY, typeString, options),
        defaultValue};
  }

  /**
   * Create a new publisher to the topic.
   *
   * The publisher is only active as long as the returned object
   * is not destroyed.
   *
   * @note It is not possible to publish two different data types to the same
   *     topic. Conflicts between publishers are typically resolved by the
   *     server on a first-come, first-served basis. Any published values that
   *     do not match the topic's data type are dropped (ignored). To determine
   *     if the data type matches, use the appropriate Topic functions.
   *
   * @param options publish options
   * @return publisher
   */
  [[nodiscard]]
  PublisherType Publish(const PubSubOptions& options = DEFAULT_PUB_SUB_OPTIONS) {
    return StringArrayPublisher{
        ::wpi::nt::Publish(m_handle, NT_STRING_ARRAY, "string[]", options)};
  }

  /**
   * Create a new publisher to the topic, with type string and initial
   * properties.
   *
   * The publisher is only active as long as the returned object
   * is not destroyed.
   *
   * @note It is not possible to publish two different data types to the same
   *     topic. Conflicts between publishers are typically resolved by the
   *     server on a first-come, first-served basis. Any published values that
   *     do not match the topic's data type are dropped (ignored). To determine
   *     if the data type matches, use the appropriate Topic functions.
   *
   * @param typeString type string
   * @param properties JSON properties
   * @param options publish options
   * @return publisher
   */
  [[nodiscard]]
  PublisherType PublishEx(std::string_view typeString,
    const wpi::util::json& properties, const PubSubOptions& options = DEFAULT_PUB_SUB_OPTIONS) {
    return StringArrayPublisher{
        ::wpi::nt::PublishEx(m_handle, NT_STRING_ARRAY, typeString, properties, options)};
  }

  /**
   * Create a new entry for the topic.
   *
   * Entries act as a combination of a subscriber and a weak publisher. The
   * subscriber is active as long as the entry is not destroyed. The publisher
   * is created when the entry is first written to, and remains active until
   * either Unpublish() is called or the entry is destroyed.
   *
   * @note It is not possible to use two different data types with the same
   *     topic. Conflicts between publishers are typically resolved by the
   *     server on a first-come, first-served basis. Any published values that
   *     do not match the topic's data type are dropped (ignored), and the entry
   *     will show no new values if the data type does not match. To determine
   *     if the data type matches, use the appropriate Topic functions.
   *
   * @param defaultValue default value used when a default is not provided to a
   *        getter function
   * @param options publish and/or subscribe options
   * @return entry
   */
  [[nodiscard]]
  EntryType GetEntry(ParamType defaultValue,
                     const PubSubOptions& options = DEFAULT_PUB_SUB_OPTIONS) {
    return StringArrayEntry{
        ::wpi::nt::GetEntry(m_handle, NT_STRING_ARRAY, "string[]", options),
        defaultValue};
  }
  /**
   * Create a new entry for the topic, with specific type string.
   *
   * Entries act as a combination of a subscriber and a weak publisher. The
   * subscriber is active as long as the entry is not destroyed. The publisher
   * is created when the entry is first written to, and remains active until
   * either Unpublish() is called or the entry is destroyed.
   *
   * @note It is not possible to use two different data types with the same
   *     topic. Conflicts between publishers are typically resolved by the
   *     server on a first-come, first-served basis. Any published values that
   *     do not match the topic's data type are dropped (ignored), and the entry
   *     will show no new values if the data type does not match. To determine
   *     if the data type matches, use the appropriate Topic functions.
   *
   * @param typeString type string
   * @param defaultValue default value used when a default is not provided to a
   *        getter function
   * @param options publish and/or subscribe options
   * @return entry
   */
  [[nodiscard]]
  EntryType GetEntryEx(std::string_view typeString, ParamType defaultValue,
                       const PubSubOptions& options = DEFAULT_PUB_SUB_OPTIONS) {
    return StringArrayEntry{
        ::wpi::nt::GetEntry(m_handle, NT_STRING_ARRAY, typeString, options),
        defaultValue};
  }

};

inline StringArrayTopic StringArraySubscriber::GetTopic() const {
  return StringArrayTopic{::wpi::nt::GetTopicFromHandle(m_subHandle)};
}

inline StringArrayTopic StringArrayPublisher::GetTopic() const {
  return StringArrayTopic{::wpi::nt::GetTopicFromHandle(m_pubHandle)};
}

inline StringArrayTopic StringArrayEntry::GetTopic() const {
  return StringArrayTopic{::wpi::nt::GetTopicFromHandle(m_subHandle)};
}

}  // namespace wpi::nt
